import numpy as np
from keras.utils import Sequence
from random import Random
from data_generator_utils import generate_pair_images


class DataGenerator(Sequence):
    def __init__(self, data_set, batch_size=20, img_size=(240, 320), channels=1, scale_min=0.9, scale_max=1.1,
                 rotate_min=-90, rotate_max=90, tx_min=-0.25, tx_max=0.25, ty_min=-0.25, ty_max=0.25):
        self.data_set = data_set
        self.batch_size = batch_size
        self.img_size = img_size
        self.channels = channels
        self.scale_min = scale_min
        self.scale_max = scale_max
        self.rotate_min = rotate_min
        self.rotate_max = rotate_max
        self.tx_min = tx_min
        self.tx_max = tx_max
        self.ty_min = ty_min
        self.ty_max = ty_max

    def __len__(self):
        return int(np.ceil(self.data_set.numImages / float(self.batch_size)))

    def __getitem__(self, index):
        X = []
        y = []
        b_start = max(index * self.batch_size, 0)
        b_end = min((index + 1) * self.batch_size, self.data_set.numImages)
        r = Random(0)
        for i in range(b_start, b_end):
            image_processed, hog_vector = generate_pair_images(self.data_set.get_image(i), r, show_info=False,
                                                               w=self.img_size[1], h=self.img_size[0])
            X.append(image_processed)
            y.append(hog_vector)

        X_train = np.array(X)
        X_train = np.expand_dims(X_train, axis=3)

        Y_train = np.array(y)

        return X_train, Y_train
