import plaidml.keras
import time
from dataset import DataSet

plaidml.keras.install_backend()
from swish import Swish


def test_model(model_name, epochs=10):
    data_set1 = DataSet('DATASETS/DATASET1.TXT')
    data_set2 = DataSet('DATASETS/DATASET2.TXT')
    print('[[DATASETS LOADED ]]\n\n')

    # Create three data generators
    print('[[ CREATING DATA GENERATORS ]]')
    from data_generator import DataGenerator
    data_generator1 = DataGenerator(data_set1, img_size=(240, 320), batch_size=4)
    data_generator2 = DataGenerator(data_set2, img_size=(240, 320), batch_size=4)

    from model_000 import Model #TODO en este punto se define el modelo
    model = Model(show_info=True, w=320, h=240, filtres_sortida=6144, name=model_name)

    from neural_network import NeuralNetwork
    neural_network = NeuralNetwork(model)
    neural_network.train_neural_network(data_generator1, val_generator=data_generator2, epochs=epochs)
    # neural_network.plot_training_history()

    neural_network.save()


def eval_model(model_name):
    from data_generator import DataGenerator
    data_set3 = DataSet('DATASETS/DATASET3.TXT')
    data_generator3 = DataGenerator(data_set3, batch_size=4)

    from neural_network import NeuralNetwork
    neural_network_loaded = NeuralNetwork()
    neural_network_loaded.load(model_name)

    neural_network_loaded.plot_training_history()
    neural_network_loaded.evaluate(data_generator3)

    from neural_network_eval import NeuralNetworkEval
    data_set = DataSet('DATASETS/DATASET3.TXT')
    evalua = NeuralNetworkEval(neural_network_loaded, data_set)

    print('[[ COMPUTING AND PLOTTING HIT RATIO EVOLUTION ]]')
    hit_ratio, auc = evalua.compute_hitratio_evolution()
    print("hit_ratio: ", hit_ratio)
    print("auc: ", auc)
    evalua.plot_hit_ratio_evolution()
    print('[[ HIT RATIO EVOLUTION COMPUTED AND PLOTTED ]]')

    print('[[ COMPUTING AND PRINTING FULL STATS ]]')
    tp, fp, tn, fn, tdist, tloops = evalua.compute_fullstats()
    print('[[ FULL STATS COMPUTED AND PRINTED ]]')

    print("Xarxa Neruronal detecta bucle determinat en la posició: ", evalua.check_one_query())


def check_one_query():
    from neural_network import NeuralNetwork
    from neural_network_eval import NeuralNetworkEval
    from os import listdir
    from os.path import isfile, splitext

    data_set = DataSet('DATASETS/DATASET3.TXT')
    neural_network_loaded = NeuralNetwork()

    files = [f for f in listdir('.') if isfile(f) and f.endswith(".h5")]
    for f in files:
        filename = splitext(f)[0]

        neural_network_loaded.load(filename)
        evalua = NeuralNetworkEval(neural_network_loaded, data_set)

        print("Xarxa Neruronal detecta bucle a la posició", filename, evalua.check_one_query())


time_start = time.time()

name = "model_000_v4" #FIXME CAUTION!!!!!!!
test_model(name, epochs=30)
eval_model(name)
#check_one_query()

time_end = time.time() - time_start
print("End time " + str(time_end))
